// @ts-check

const CommandBase = require("./CommandBase");

class NVMeDiskDetection extends CommandBase
{
	constructor(objConfig = {})
	{
		super(objConfig);
	}

	static get command()
	{
		return "lsblk -b -P -o KNAME,TYPE,VENDOR,TRAN,MODEL,SERIAL,SIZE,ROTA | grep \"TYPE=\\\"disk\\\"\" | grep nvme";
	}

	/**
	 * Parses the raw output from the command, must me runned after execCommand
	 * 
	 * @param {string} [strResult=null]
	 * @returns {Promise<Object>} _objParsedCommandOutput
	 */
	async _parseCommandOutput(strResult = null)
	{
		if(strResult)
		{
			this._strRawCommandOutput = strResult;
		}

		if(!this._strRawCommandOutput || this._strRawCommandOutput.trim() === "")
		{
			throw new Error(`Empty output string for command ${this.constructor.name}.`);
		}

		const arrNVMeDiskInfo = this._strRawCommandOutput.split("\n");
		arrNVMeDiskInfo.forEach((strDiskInfo, nIndex) =>
		{
			const objDisk = {};
			const arrDiskPropertyRaw = strDiskInfo.split("\" ");

			arrDiskPropertyRaw.forEach((strDiskPropertyRaw, nIndex) => 
			{
				const arrDiskProperty = strDiskPropertyRaw.split("=\"");
				objDisk[arrDiskProperty[0]] = arrDiskProperty[1];
			});

			this._objParsedCommandOutput[objDisk["KNAME"]] = {
				size: objDisk["SIZE"],
				vendor: objDisk["VENDOR"]
			};
		});

		return this._objParsedCommandOutput;
	}

	get parsedOutput()
	{
		return this._objParsedCommandOutput;
	}

	static get osPlatform()
	{
		return "linux";
	}

	async test()
	{
		
	}
}

module.exports = new NVMeDiskDetection({bMustBeFiltered: false, bMustBeParsed: true});
