const path = require("path");
const fs = require("fs-extra");
const assert = require("assert");

const Commands = require("./../Commands");
const ServerInfoGatherer = require("./../ServerInfoGatherer");

class AllTests
{
	constructor()
	{

	}

	async runAllTests()
	{
		await this.outputCollectionTests();
		await this.platformSpecificTests();
		await this.propertiesExistanceTests();
		await this.selfTests();
	}

	/* 
	 * Testing the parsing and filtering functions on each command by feeding them
	 * saved outputs and comparing the results with some verified ones
	*/
	async outputCollectionTests()
	{
		const strRootOutputCollectionDirPath = path.join(__dirname, "commandsOutputCollection");

		for(let strDirName of await fs.readdir(strRootOutputCollectionDirPath))
		{
			const strEnvironmentDirPath = path.join(strRootOutputCollectionDirPath, strDirName);

			if((await fs.stat(strEnvironmentDirPath)).isDirectory())
			{
				const strRawOutputDirPath = path.join(strEnvironmentDirPath, "rawOutput");
				const strParsedOutputDirPath = path.join(strEnvironmentDirPath, "parsedOutput");
				const strFilteredOutputDirPath = path.join(strEnvironmentDirPath, "filteredOutput");

				if(
					(await fs.stat(strRawOutputDirPath)).isDirectory()
					&& (await fs.stat(strParsedOutputDirPath)).isDirectory()
					&& (await fs.stat(strFilteredOutputDirPath)).isDirectory()
				)
				{
					for(let strFileName of await fs.readdir(strRawOutputDirPath))
					{
						const strRawFilePath = path.join(strRawOutputDirPath, strFileName);
						const strParsedFilePath = path.join(strParsedOutputDirPath, strFileName);
						const strFilteredFilePath = path.join(strFilteredOutputDirPath, strFileName);
						
						if(path.extname(strFileName).toLowerCase() !== ".txt")
						{
							console.log(`---> Skipping ${path.relative(strRawOutputDirPath, strRawFilePath)} because file extension is not .txt`);
							continue;
						}

						if(
							(await fs.stat(strRawFilePath)).isFile()
							&& (await fs.stat(strParsedFilePath)).isFile()
							&& (await fs.stat(strFilteredFilePath)).isFile()
						)
						{
							const strRawOutput = await fs.readFile(strRawFilePath, "utf8");
							const strStoredParsedOutput = await fs.readFile(strParsedFilePath, "utf8");
							// const strFilteredOutput = await fs.readFile(strFilteredFilePath);
							const strCommandName = (strFileName.split("_")[1]).slice(0, -4);
							
							try
							{
								let strGeneratedParsedOutput = await Commands[strCommandName]._parseCommandOutput(strRawOutput);
		
								strGeneratedParsedOutput = typeof strGeneratedParsedOutput === "object"
									? JSON.stringify(strGeneratedParsedOutput) 
									: strGeneratedParsedOutput.toString();

								// Parsing test
								assert.strictEqual(strGeneratedParsedOutput, strStoredParsedOutput, `After parsing the raw output for ${strCommandName} command, the result doesn't match with the stored one`);
								console.log(`---> Success on testing command [${strCommandName}]`);
	
								// // Filtering test
								// assert.strictEqual(strGeneratedFilteredOutput, strFilteredOutput, "After filtering the parsed output the result doesn't match with the stored one");
							}
							catch(error)
							{
								console.log(`---> Failed on testing command [${strCommandName}]`);
								console.log(error);
							}
						}
					}
				}
			}
		}	
	}

	async platformSpecificTests()
	{

	}

	async selfTests()
	{
		await Commands.Lshw.test();
		await Commands.HBA.test();
		await Commands.CCMcfg.test();
		await Commands.NVMe.test();
		await Commands.NVMeDiskDetection.test();
		await Commands.DiskDetection.test();
		await Commands.EFIbootmgr.test();
		await Commands.NetworkInterfaces.test();
	}

	async propertiesExistanceTests()
	{
		const arrRequiredFields = [
			"server_uuid",
			"server_product_name",
			"server_vendor",
			"server_serial_number",

			"server_bios_vendor",
			"server_bios_version",

			"server_processor_count",
			"server_processor_core_count",
			"server_processor_name",
			"server_processor_core_mhz",
			"server_processor_cpu_mark",
			"server_processor_threads",

			"server_ram_gbytes",

			"server_network_total_capacity_mbps",

			"server_disk_count",
			"server_disk_size_gb",
			"server_disk_type",
			"server_disk_vendor",
			"server_disk_model",
			"server_disk_serial"
		];

		const objTestGatherer = new ServerInfoGatherer();
		const objResults = await objTestGatherer.gatherAllInfo();
		const arrPropertiesNames = [];

		arrPropertiesNames.push(...Object.keys(objResults));

		const arrMissingProperties = arrRequiredFields.filter(strPropertyName => !arrPropertiesNames.includes(strPropertyName));

		if(arrMissingProperties.length)
		{
			throw new Error(`There are missing the following properties from the information object: [${arrMissingProperties}]`);
		}
	}
}

module.exports = AllTests;
